@@ -1,7 +1,13 @@
+from nested_admin.nested import NestedModelAdmin, NestedTabularInline
+
+try:
+    from django.db.models import JSONField
+except ImportError:  # TODO: Remove when dropping Django releases < 3.1
+    from django.contrib.postgres.fields import JSONField
+import admin_thumbnails
 from admin_ordering.admin import OrderableAdmin
 from django.contrib import admin, messages
 from django.contrib.gis.admin import OSMGeoAdmin
-from django.contrib.postgres import fields
 from django.utils.translation import gettext_lazy as _
 from django_json_widget.widgets import JSONEditorWidget
 from django_object_actions import DjangoObjectActions
@@ -10,16 +16,19 @@ from reversion.admin import VersionAdmin
 from sorl.thumbnail.admin import AdminInlineImageMixin
 
 from . import models, forms
-from .properties.schema import sync_layer_schema, sync_ui_schema, clean_properties_not_in_schema_or_null
+from .properties.schema import sync_layer_schema, sync_ui_schema, clean_properties_not_in_schema_or_null, \
+    sync_properties_in_tiles
 
 
+@admin_thumbnails.thumbnail('pictogram')
 class CrudGroupViewAdmin(OrderableAdmin, VersionAdmin):
     ordering_field = "order"
     list_editable = ["order"]
-    list_display = ['name', 'order', 'pictogram']
+    list_display = ['name', 'order', 'pictogram_thumbnail']
 
 
-class FeatureDisplayGroupTabularInline(OrderableAdmin, admin.TabularInline):
+@admin_thumbnails.thumbnail('pictogram')
+class FeatureDisplayGroupTabularInline(OrderableAdmin, NestedTabularInline):
     ordering_field = "order"
     classes = ('collapse', )
     verbose_name = _('Display group')
@@ -28,7 +37,7 @@ class FeatureDisplayGroupTabularInline(OrderableAdmin, admin.TabularInline):
     extra = 0
 
 
-class ExtraLayerStyleInLine(admin.TabularInline):
+class ExtraLayerStyleInLine(NestedTabularInline):
     classes = ('collapse', )
     verbose_name = _('Extra layer style')
     verbose_name_plural = _('Extra layer styles')
@@ -36,11 +45,20 @@ class ExtraLayerStyleInLine(admin.TabularInline):
     form = forms.ExtraLayerStyleForm
     extra = 0
     formfield_overrides = {
-        fields.JSONField: {'widget': JSONEditorWidget},
+        JSONField: {'widget': JSONEditorWidget},
     }
 
 
-class CrudPropertyInline(OrderableAdmin, admin.TabularInline):
+@admin_thumbnails.thumbnail('pictogram')
+class PropertyEnumInline(NestedTabularInline):
+    model = models.PropertyEnum
+    extra = 0
+    classes = ('collapse', )
+    verbose_name = _("Value")
+    verbose_name_plural = _("Available values. Let empty if you want to let free input.")
+
+
+class CrudPropertyInline(OrderableAdmin, NestedTabularInline):
     ordering_field = "order"
     classes = ('collapse', )
     verbose_name = _("Feature property")
@@ -49,21 +67,23 @@ class CrudPropertyInline(OrderableAdmin, admin.TabularInline):
     form = forms.CrudPropertyForm
     extra = 0
     formfield_overrides = {
-        fields.JSONField: {'widget': JSONEditorWidget(height=200)},
+        JSONField: {'widget': JSONEditorWidget(height=200)},
     }
+    inlines = [PropertyEnumInline, ]
 
 
-class CrudViewAdmin(OrderableAdmin, DjangoObjectActions, VersionAdmin):
+@admin_thumbnails.thumbnail('pictogram')
+class CrudViewAdmin(OrderableAdmin, DjangoObjectActions, VersionAdmin, NestedModelAdmin):
     ordering_field = "order"
     list_editable = ["order"]
     filter_horizontal = ('default_list_properties', )
     form = forms.CrudViewForm
-    list_display = ['name', 'group', 'order', 'pictogram']
+    list_display = ['name', 'group', 'order', 'pictogram_thumbnail']
     list_filter = ['group', ]
-    inlines = [CrudPropertyInline, FeatureDisplayGroupTabularInline, ExtraLayerStyleInLine]
-    readonly_fields = ['ui_schema']
+    inlines = [FeatureDisplayGroupTabularInline, CrudPropertyInline, ExtraLayerStyleInLine]
+    readonly_fields = ('ui_schema', )
     fieldsets = (
-        (None, {'fields': (('name', 'object_name', 'object_name_plural', 'layer'), ('group', 'order', 'pictogram'))}),
+        (None, {'fields': (('name', 'object_name', 'object_name_plural', 'layer'), ('group', 'order', 'pictogram', 'pictogram_thumbnail'))}),
         (_('UI schema & properties'), {
             'fields': ('default_list_properties', 'feature_title_property', 'ui_schema'),
             'classes': ('collapse', )
@@ -79,7 +99,7 @@ class CrudViewAdmin(OrderableAdmin, DjangoObjectActions, VersionAdmin):
     )
 
     formfield_overrides = {
-        fields.JSONField: {'widget': JSONEditorWidget},
+        JSONField: {'widget': JSONEditorWidget},
     }
 
     def get_readonly_fields(self, request, obj=None):
@@ -105,7 +125,14 @@ class CrudViewAdmin(OrderableAdmin, DjangoObjectActions, VersionAdmin):
     clean_feature_properties.label = _("Clean features with schema")
     clean_feature_properties.short_description = _("Clean feature properties not in generated layer schema.")
 
-    change_actions = ('sync_schemas', 'clean_feature_properties')
+    def sync_tile_content(self, request, obj):
+        sync_properties_in_tiles(obj)
+        messages.success(request, _("Properties in tiles have been synced."))
+
+    sync_tile_content.label = _("Define properties in tiles.")
+    sync_tile_content.short_description = _("Define property marked as included in tile in layer definition..")
+
+    change_actions = ('sync_schemas', 'clean_feature_properties', 'sync_tile_content')
 
 
 class LayerExtraGeomInline(admin.TabularInline):
@@ -118,7 +145,7 @@ class CrudLayerAdmin(VersionAdmin):
     list_filter = ('geom_type', 'layer_groups')
     search_fields = ('pk', 'name')
     formfield_overrides = {
-        fields.JSONField: {'widget': JSONEditorWidget},
+        JSONField: {'widget': JSONEditorWidget},
     }
     inlines = [LayerExtraGeomInline, ]
     readonly_fields = ('schema', )  # schema is managed with crud view properties
@@ -156,10 +183,11 @@ class CrudFeatureAdmin(VersionAdmin, OSMGeoAdmin):
     search_fields = ('pk', 'identifier', )
     inlines = (FeatureExtraGeomInline, FeaturePictureInline, FeatureAttachmentInline)
     formfield_overrides = {
-        fields.JSONField: {'widget': JSONEditorWidget},
+        JSONField: {'widget': JSONEditorWidget},
     }
 
 
+@admin_thumbnails.thumbnail('pictogram')
 class AttachmentCategoryAdmin(VersionAdmin):
-    list_display = ('pk', 'name', 'pictogram')
+    list_display = ('pk', 'name', 'pictogram_thumbnail')
     search_fields = ('pk', 'name', )
