@@ -7,6 +7,7 @@ from django.utils.translation import gettext as _
 from geostore.models import FeatureExtraGeom
 
 from . import models
+from .models import CrudView
 
 
 def parse_geometry_file(geom_file):
@@ -14,13 +15,7 @@ def parse_geometry_file(geom_file):
     temp.write(geom_file.read())
     temp.close()
     ds = DataSource(temp.name)
-
-    if len(ds) == 0:
-        return
-    layer = ds[0]
-    if len(layer) == 0:
-        return
-    geom = layer[0].geom.clone()
+    geom = ds[0][0].geom.clone()
     geom.coord_dim = 2
     return geom.geos
 
@@ -58,6 +53,10 @@ class CrudViewForm(forms.ModelForm):
             # limit choices to available (linked by crud view)
             self.fields['default_list_properties'].queryset = self.instance.list_available_properties.all()
             self.fields['feature_title_property'].queryset = self.instance.list_available_properties.all()
+            # can only select a layer not used by a crud view
+        else:
+            self.fields['layer'].queryset = self.fields['layer'].queryset.\
+                exclude(pk__in=CrudView.objects.values_list('layer_id', flat=True))
 
     class Meta:
         model = models.CrudView
