@@ -1,7 +1,14 @@
 from copy import deepcopy
 
 from django.contrib.gis.db.models import Extent
-from django.contrib.postgres.fields import JSONField
+from django.core.exceptions import ValidationError
+from django.db.models import FloatField, CharField, IntegerField
+from django.db.models.functions import Cast
+
+try:
+    from django.db.models import JSONField
+except ImportError:  # TODO: Remove when dropping Django releases < 3.1
+    from django.contrib.postgres.fields import JSONField
 from django.contrib.postgres.indexes import GinIndex
 from django.db import models
 from django.utils.functional import cached_property
@@ -18,8 +25,10 @@ from .validators import validate_schema_property
 
 
 class CrudModelMixin(models.Model):
-    name = models.CharField(max_length=100, unique=True, help_text=_("Display name in left menu"))
-    order = models.PositiveSmallIntegerField(help_text=_("Order entry in left menu"), db_index=True)
+    name = models.CharField(max_length=100, unique=True, help_text=_("Display name in left menu"),
+                            verbose_name=_('Name'))
+    order = models.PositiveSmallIntegerField(verbose_name=_("Order"),
+                                             help_text=_("Order entry in left menu"), db_index=True)
 
     def __str__(self):
         return self.name
@@ -32,7 +41,7 @@ class CrudGroupView(CrudModelMixin):
     """
     Used to defined group of view in CRUD
     """
-    pictogram = models.ImageField(upload_to='crud/groups/pictograms', null=True, blank=True,
+    pictogram = models.ImageField(upload_to='terra_geocrud/groups/pictograms', null=True, blank=True,
                                   help_text=_("Picto displayed in left menu"))
 
     class Meta:
@@ -45,24 +54,32 @@ class CrudView(FormSchemaMixin, MapStyleModelMixin, CrudModelMixin):
     """
     Used to defined ad layer's view in CRUD
     """
-    object_name = models.CharField(max_length=100, default="", blank=True, null=False)
-    object_name_plural = models.CharField(max_length=100, default="", blank=True, null=False)
-    group = models.ForeignKey(CrudGroupView, on_delete=models.SET_NULL, related_name='crud_views',
+    object_name = models.CharField(verbose_name=_("Singular object name"), max_length=100, default="",
+                                   blank=True, null=False)
+    object_name_plural = models.CharField(verbose_name=_("Plural object name"), max_length=100, default="",
+                                          blank=True, null=False)
+    group = models.ForeignKey(CrudGroupView, verbose_name=_("Group"), on_delete=models.SET_NULL,
+                              related_name='crud_views',
                               null=True, blank=True, help_text=_("Group this entry in left menu"))
-    layer = models.OneToOneField('geostore.Layer', on_delete=models.CASCADE, related_name='crud_view')
+    layer = models.OneToOneField('geostore.Layer', on_delete=models.CASCADE, related_name='crud_view',
+                                 verbose_name=_("Layer"))
     templates = models.ManyToManyField('template_model.Template', related_name='crud_views', blank=True,
+                                       verbose_name=_("Document templates"),
                                        help_text=_("Available templates for layer features document generation"))
-    pictogram = models.ImageField(upload_to='crud/views/pictograms', null=True, blank=True,
+    pictogram = models.ImageField(upload_to='terra_geocrud/views/pictograms', null=True, blank=True,
                                   help_text=_("Picto displayed in left menu"))
-    map_style = JSONField(default=dict, blank=True, help_text=_("Custom mapbox style for this entry"))
+    map_style = JSONField(default=dict, blank=True, help_text=_("Custom mapbox style for this entry"),
+                          verbose_name=_("Map style"))
     ui_schema = JSONField(default=dict, blank=True, editable=False,
                           help_text=_("""Custom ui:schema style for this entry.
                                          https://react-jsonschema-form.readthedocs.io/en/latest/form-customization/"""))
     # WARNING: settings is only used to wait for model definition
     settings = JSONField(default=dict, blank=True)
-    default_list_properties = models.ManyToManyField('CrudViewProperty', blank=True, related_name='used_by_list',
+    default_list_properties = models.ManyToManyField('CrudViewProperty', verbose_name=_("Properties in feature list"),
+                                                     blank=True, related_name='used_by_list',
                                                      help_text=_("Schema properties used in API list by default."),)
     feature_title_property = models.ForeignKey('CrudViewProperty', null=True, on_delete=models.SET_NULL,
+                                               verbose_name=_("Title property"),
                                                help_text=_("Schema property used to define feature title."),
                                                related_name='used_by_title', blank=True)
     visible = models.BooleanField(default=True, db_index=True, help_text=_("Keep visible if ungrouped."))
@@ -118,7 +135,7 @@ class FeaturePropertyDisplayGroup(models.Model):
     order = models.PositiveSmallIntegerField(default=0, db_index=True)
     label = models.CharField(max_length=50)
     slug = models.SlugField(blank=True, editable=False)
-    pictogram = models.ImageField(upload_to='crud/feature_display_group/pictograms', null=True, blank=True)
+    pictogram = models.ImageField(upload_to='terra_geocrud/property_groups/pictograms', null=True, blank=True)
 
     def __str__(self):
         return self.label
@@ -160,7 +177,7 @@ class FeaturePropertyDisplayGroup(models.Model):
 
 class AttachmentCategory(models.Model):
     name = models.CharField(unique=True, max_length=255)
-    pictogram = models.ImageField(upload_to='crud/attachments_category/pictograms', null=True, blank=True)
+    pictogram = models.ImageField(upload_to='terra_geocrud/attachments_categories/pictograms', null=True, blank=True)
 
     def __str__(self):
         return self.name
@@ -251,7 +268,8 @@ class CrudViewProperty(models.Model):
     key = models.SlugField()
     json_schema = JSONField(blank=False, null=False, default=dict, validators=[validate_schema_property])
     ui_schema = JSONField(blank=True, null=False, default=dict)
-    required = models.BooleanField(default=False)
+    include_in_tile = models.BooleanField(default=False, db_index=True)
+    required = models.BooleanField(default=False, db_index=True)
     order = models.PositiveSmallIntegerField(default=0, db_index=True)
 
     class Meta:
@@ -276,11 +294,55 @@ class CrudViewProperty(models.Model):
                                   self.json_schema.get('title',
                                                        self.key.capitalize()))
 
-    def delete(self, **kwargs):
-        super().delete(**kwargs)
+    @cached_property
+    def full_json_schema(self):
+        """
+        Generate full json schema by adding custom conf to store schema.
+        All keys defined in json_schema column are kept if already present.
+        """
+        output_field = CharField()
+        if self.json_schema.get("type") == "number" or (
+                self.json_schema.get("type") == "array" and self.json_schema.get("items").get("type") == "number"):
+            # final values should be float
+            output_field = FloatField()
+        elif self.json_schema.get("type") == "integer" or (
+                self.json_schema.get("type") == "array" and self.json_schema.get("items").get("type") == "integer"):
+            # final values should be integer
+            output_field = IntegerField()
+        values = self.values.all().annotate(final_value=Cast('value', output_field=output_field))
+        if values:
+            json_schema = deepcopy(self.json_schema)
+            if self.json_schema.get('type') != "array":
+                # in non array properties, enum are defined in enum key
+                json_schema.setdefault('enum', list(values.values_list('final_value', flat=True)))
+            else:
+                # in array, enum values are defined in 'items__enum' key
+                json_schema['items'].setdefault('enum', list(values.values_list('final_value', flat=True)))
+            return json_schema
+        return self.json_schema
 
-        # delete feature property keys
-        fs = self.view.layer.features.all()
-        for f in fs:
-            f.properties.pop(self.key, None)
-            f.save()
+
+class PropertyEnum(models.Model):
+    value = models.CharField(max_length=250, help_text=_("Value should always be casted in property type."))
+    pictogram = models.ImageField(upload_to='terra_geocrud/enums/pictograms', null=True, blank=True,
+                                  help_text=_("Picto. associated to value."))
+    property = models.ForeignKey(CrudViewProperty, on_delete=models.CASCADE, related_name='values')
+
+    def clean(self):
+        try:
+            if self.property.json_schema.get('type') == 'integer':
+                int(self.value)
+            elif self.property.json_schema.get('type') == 'number':
+                float(self.value)
+        except ValueError:
+            raise ValidationError(
+                _(f"Value '{self.value}' should be casted as property type ({self.property.json_schema.get('type')})")
+            )
+
+    def __str__(self):
+        return self.value
+
+    class Meta:
+        unique_together = (
+            ('value', 'property'),
+        )
