@@ -1,13 +1,14 @@
 import json
 from collections import OrderedDict
 from copy import deepcopy
-from datetime import datetime
 from pathlib import Path
 
 from django.template.defaultfilters import date
+from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
+from geostore import settings as geostore_settings
 from geostore.models import LayerExtraGeom
-from geostore.serializers import LayerSerializer, FeatureSerializer, FeatureExtraGeomSerializer
+from geostore.serializers import FeatureSerializer, FeatureExtraGeomSerializer
 from rest_framework import serializers
 from rest_framework.reverse import reverse
 from rest_framework_gis import serializers as geo_serializers
@@ -15,12 +16,12 @@ from template_model.models import Template
 
 from . import models
 from .map.styles import get_default_style
-from .properties.files import get_info_content, get_storage_file_url, \
-    get_storage_path_from_infos, store_feature_files
-from .thumbnail_backends import ThumbnailDataFileBackend
+from .properties.files import store_feature_files
+from .properties.utils import serialize_group_properties
 
+# use base serializer as defined in geostore settings. using django-geostore-routing change this value
 
-thumbnail_backend = ThumbnailDataFileBackend()
+LayerSerializer = import_string(geostore_settings.GEOSTORE_LAYER_SERIALIZER)
 
 
 class BaseUpdatableMixin(serializers.ModelSerializer):
@@ -158,56 +159,7 @@ class FeatureDisplayPropertyGroup(serializers.ModelSerializer):
             for prop in obj.group_properties.all()
         }
 
-        properties = {}
-
-        for key, value in final_properties.items():
-            data_type = 'data'
-            data = value
-            data_format = feature.layer.schema.get('properties', {}).get(key, {}).get('format')
-
-            if data_format == 'data-url':
-                # apply special cases for files
-                data_type = 'file'
-                data = {"url": None}
-                if value:
-                    # generate / get thumbnail for image
-                    try:
-                        # try to get file info from "data:image/png;xxxxxxxxxxxxx" data
-                        infos, content = get_info_content(value)
-                        storage_file_path = get_storage_path_from_infos(infos)
-                        data['url'] = get_storage_file_url(storage_file_path)
-
-                        if infos and infos.split(';')[0].split(':')[1].split('/')[0] == 'image':
-                            # apply special cases for images
-                            data_type = 'image'
-                            try:
-                                data.update({
-                                    "thumbnail": thumbnail_backend.get_thumbnail(storage_file_path,
-                                                                                 "500x500",
-                                                                                 upscale=False).url
-                                })
-                            except ValueError:
-                                pass
-                    except IndexError:
-                        pass
-            elif data_format == "date":
-                data_type = 'date'
-                try:
-                    value_date = datetime.fromisoformat(str(value))
-                    data = date(value_date, 'SHORT_DATE_FORMAT')
-                except ValueError:
-                    pass
-
-            properties.update({key: {
-                "display_value": data,
-                "type": data_type,
-                "title": feature.layer.get_property_title(key),
-                "value": feature.properties.get(key),
-                "schema": feature.layer.schema.get('properties', {}).get(key),
-                "ui_schema": feature.layer.crud_view.ui_schema.get(key, {})
-            }})
-
-        return properties
+        return serialize_group_properties(feature, final_properties)
 
     class Meta:
         model = models.FeaturePropertyDisplayGroup
@@ -373,56 +325,7 @@ class CrudFeatureDetailSerializer(BaseUpdatableMixin, FeatureSerializer):
                 prop.key: obj.properties.get(prop.key)
                 for prop in remained_properties
             }
-            properties = {}
-
-            for key, value in final_properties.items():
-                data_type = 'data'
-                data = value
-                data_format = obj.layer.schema.get('properties', {}).get(key, {}).get('format')
-
-                if data_format == 'data-url':
-                    # apply special cases for files
-                    data_type = 'file'
-                    data = {"url": None}
-                    if value:
-                        # generate / get thumbnail for image
-                        try:
-                            # try to get file info from "data:image/png;xxxxxxxxxxxxx" data
-                            infos, content = get_info_content(value)
-                            storage_file_path = get_storage_path_from_infos(infos)
-
-                            data['url'] = get_storage_file_url(storage_file_path)
-
-                            if infos.split(';')[0].split(':')[1].split('/')[0] == 'image':
-                                # apply special cases for images
-                                data_type = 'image'
-                                try:
-                                    data.update({
-                                        "thumbnail": thumbnail_backend.get_thumbnail(storage_file_path,
-                                                                                     "500x500",
-                                                                                     upscale=False).url
-                                    })
-                                except ValueError:
-                                    pass
-
-                        except IndexError:
-                            pass
-                elif data_format == "date":
-                    data_type = 'date'
-                    try:
-                        value_date = datetime.fromisoformat(str(value))
-                        data = date(value_date, 'SHORT_DATE_FORMAT')
-                    except ValueError:
-                        pass
-
-                properties.update({key: {
-                    "display_value": data,
-                    "type": data_type,
-                    "title": obj.layer.get_property_title(key),
-                    "value": obj.properties.get(key),
-                    "schema": obj.layer.schema.get('properties', {}).get(key),
-                    "ui_schema": obj.layer.crud_view.ui_schema.get(key, {})
-                }})
+            properties = serialize_group_properties(obj, final_properties)
 
             results['__default__'] = {
                 "title": "",
