@@ -1,8 +1,11 @@
 import json
+import os
 from tempfile import TemporaryDirectory
 from unittest import mock
 
+from django.conf import settings
 from django.contrib.gis.geos import GeometryCollection, LineString, Point
+from django.core.files.base import ContentFile
 from django.template import Context, Template
 from django.template.base import FilterExpression, Parser
 from django.template.exceptions import TemplateSyntaxError
@@ -10,6 +13,7 @@ from django.test import TestCase
 from django.test.utils import override_settings
 from rest_framework.reverse import reverse
 from rest_framework.test import APITestCase
+from terra_geocrud.tests.settings import PICTURES_PATH
 
 from . import factories
 from .settings import FEATURE_PROPERTIES, LAYER_SCHEMA, SMALL_PICTURE
@@ -18,8 +22,8 @@ from geostore.models import Feature, FeatureExtraGeom, LayerExtraGeom
 from geostore import GeometryTypes
 
 from mapbox_baselayer.models import BaseLayerTile, MapBaseLayer
-from terra_geocrud.models import ExtraLayerStyle, CrudViewProperty
-from terra_geocrud.templatetags.map_tags import MapImageLoaderNodeURL, stored_image_base64
+from terra_geocrud.models import ExtraLayerStyle, CrudViewProperty, PropertyEnum
+from terra_geocrud.templatetags.map_tags import MapImageLoaderURLODTNode, stored_image_base64
 from terra_geocrud import settings as app_settings
 from ..properties.schema import sync_layer_schema
 
@@ -57,7 +61,7 @@ class MapImageUrlLoaderTestCase(TestCase):
         self.crud_view_point.templates.add(self.template)
         self.crud_view_line.templates.add(self.template)
 
-        self.node = MapImageLoaderNodeURL('http://mbglrenderer/render')
+        self.node = MapImageLoaderURLODTNode('http://mbglrenderer/render')
 
         self.token_mapbox = app_settings.TERRA_GEOCRUD.get('map', {}).get('mapbox_access_token')
 
@@ -145,15 +149,22 @@ class StyleMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 
         dict_style = {
             "version": 8,
-            "sources":
-                {"baselayercustom": {"type": "raster",
-                                     "tiles": ["test.test"],
-                                     'attribution': '',
-                                     'tileSize': 512,
-                                     "minzoom": 0,
-                                     "maxzoom": 22, }},
-            "layers": [
-                {"id": "baselayercustom-background", "type": "raster", "source": "baselayercustom"}]
+            "sources": {
+                "baselayercustom": {
+                    "type": "raster",
+                    "tiles": ["test.test"],
+                    'attribution': '',
+                    'tileSize': 512,
+                    "minzoom": 0,
+                    "maxzoom": 22,
+                }
+            },
+            'glyphs': 'mapbox://fonts/mapbox/{fontstack}/{range}.pbf',
+            "layers": [{
+                "id": "baselayercustom-background",
+                "type": "raster",
+                "source": "baselayercustom"
+            }]
         }
 
         self.assertDictEqual(dict_style, self.node.get_style(self.line, False, [''], None))
@@ -166,15 +177,22 @@ class StyleMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 
         dict_style = {
             "version": 8,
-            "sources":
-                {"otherlayercustom": {"type": "raster",
-                                      "attribution": "",
-                                      "tileSize": 512,
-                                      "tiles": ["test.test"],
-                                      "minzoom": 0,
-                                      "maxzoom": 22}},
-            "layers": [
-                {"id": "otherlayercustom-background", "type": "raster", "source": "otherlayercustom"}]
+            "sources": {
+                "otherlayercustom": {
+                    "type": "raster",
+                    "attribution": "",
+                    "tileSize": 512,
+                    "tiles": ["test.test"],
+                    "minzoom": 0,
+                    "maxzoom": 22
+                }
+            },
+            'glyphs': 'mapbox://fonts/mapbox/{fontstack}/{range}.pbf',
+            "layers": [{
+                "id": "otherlayercustom-background",
+                "type": "raster",
+                "source": "otherlayercustom"
+            }]
         }
 
         self.assertDictEqual(dict_style, self.node.get_style(self.line, False, [''], 'otherlayercustom'))
@@ -194,11 +212,11 @@ class StyleMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 class ContextMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
     def test_get_value_context_line(self, token):
         self.maxDiff = None
-        self.node = MapImageLoaderNodeURL('http://mbglrenderer/render', data={'width': None,
-                                                                              'height': None,
-                                                                              'feature_included': None,
-                                                                              'extra_features': None,
-                                                                              'base_layer': None})
+        self.node = MapImageLoaderURLODTNode('http://mbglrenderer/render', data={'width': None,
+                                                                                 'height': None,
+                                                                                 'feature_included': None,
+                                                                                 'extra_features': None,
+                                                                                 'base_layer': None})
         style = self.node.get_data(Context({'object': self.line}))
         dict_style = {
             "version": 8,
@@ -227,11 +245,11 @@ class ContextMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
         self.maxDiff = None
         settings_terra = app_settings.TERRA_GEOCRUD
         settings_terra['MAX_ZOOM'] = 20
-        self.node = MapImageLoaderNodeURL('http://mbglrenderer/render', data={'width': None,
-                                                                              'height': None,
-                                                                              'feature_included': None,
-                                                                              'extra_features': None,
-                                                                              'base_layer': None})
+        self.node = MapImageLoaderURLODTNode('http://mbglrenderer/render', data={'width': None,
+                                                                                 'height': None,
+                                                                                 'feature_included': None,
+                                                                                 'extra_features': None,
+                                                                                 'base_layer': None})
 
         with override_settings(TERRA_GEOCRUD=settings_terra):
             style = self.node.get_data(Context({'object': self.point}))
@@ -259,11 +277,14 @@ class ContextMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 
     def test_get_value_context_line_with_extra_features(self, token):
         self.maxDiff = None
-        self.node = MapImageLoaderNodeURL('http://mbglrenderer/render', data={'width': None,
-                                                                              'height': None,
-                                                                              'feature_included': None,
-                                                                              'base_layer': None,
-                                                                              'extra_features': FilterExpression("'test'", Parser(''))})
+        self.node = MapImageLoaderURLODTNode(
+            'http://mbglrenderer/render',
+            data={'width': None,
+                  'height': None,
+                  'feature_included': None,
+                  'base_layer': None,
+                  'extra_features': FilterExpression("'test'", Parser(''))}
+        )
         style = self.node.get_data(Context({'object': self.line}))
 
         dict_style = {
@@ -307,7 +328,7 @@ class ContextMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 
 
 @mock.patch('secrets.token_hex', side_effect=['primary', 'test'])
-class RenderMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
+class RenderMapImageUrlLoaderODTTestCase(MapImageUrlLoaderTestCase):
     @mock.patch('requests.post')
     def test_image_url_loader_object(self, mocked_post, token):
         mocked_post.return_value.status_code = 200
@@ -317,12 +338,12 @@ class RenderMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
 
         rendered_template = template_to_render.render(context)
         self.assertEqual('<draw:frame draw:name="test.png" svg:width="15.0" svg:height="15.0" '
-                         'text:anchor-type="paragraph" draw:z-index="0">'
-                         '<draw:image xlink:href="Pictures/test.png" xlink:show="embed" xlink:actuate="onLoad"/>'
+                         'text:anchor-type="paragraph" draw:z-index="37">'
+                         '<draw:image xlink:href="Pictures/test.png" xlink:type="simple" xlink:show="embed" '
+                         'xlink:actuate="onLoad" draw:mime-type="image/png" />'
                          '</draw:frame>', rendered_template)
 
-    @mock.patch('requests.post')
-    def test_map_image_url_loader_usage(self, mocked_post, token):
+    def test_map_image_url_loader_usage(self, token):
         with self.assertRaises(TemplateSyntaxError) as cm:
             Template('{% load map_tags %}{% map_image_url_loader wrong_key="test" %}')
         self.assertEqual('Usage: {% map_image_url_loader width="5000" height="5000" feature_included=False '
@@ -336,19 +357,45 @@ class RenderMapImageUrlLoaderTestCase(MapImageUrlLoaderTestCase):
         self.assertEqual('', rendered_template)
 
 
+class RenderMapImageUrlLoaderPDFTestCase(MapImageUrlLoaderTestCase):
+    @mock.patch('requests.post')
+    def test_image_url_loader_object(self, mocked_post):
+        mocked_post.return_value.status_code = 200
+        mocked_post.return_value.content = SMALL_PICTURE
+        context = Context({'object': self.line})
+        template_to_render = Template('{% load map_tags %}{% image_base64_from_url %}')
+
+        rendered_template = template_to_render.render(context)
+        self.assertTrue(rendered_template.startswith("data:image/png;base64,"))
+        self.assertTrue(rendered_template.endswith("="))
+
+    def test_map_image_url_loader_usage(self):
+        with self.assertRaises(TemplateSyntaxError) as cm:
+            Template('{% load map_tags %}{% image_base64_from_url wrong_key="test" %}')
+        self.assertEqual('Usage: {% image_base64_from_url width="5000" height="5000" feature_included=False '
+                         'extra_features="feature_1" base_layer="mapbaselayer_1" %}', str(cm.exception))
+
+    def test_image_url_loader_no_object(self):
+        context = Context({'object': self.line})
+        template_to_render = Template('{% load map_tags %}{% image_base64_from_url feature_included=False %}')
+
+        rendered_template = template_to_render.render(context)
+        self.assertEqual('', rendered_template)
+
+
 class ZoomLineMapImageUrlLoaderTestCase(TestCase):
     def test_get_zoom(self):
-        node = MapImageLoaderNodeURL('http://mbglrenderer/render')
+        node = MapImageLoaderURLODTNode('http://mbglrenderer/render')
         collection = GeometryCollection(LineString([[-180, -85.06], [180, 85.06]]), srid=4326)
         self.assertEqual(0, node.get_zoom_bounds(1024, 1024, collection))
 
     def test_get_zoom_no_width(self):
-        node = MapImageLoaderNodeURL('http://mbglrenderer/render')
+        node = MapImageLoaderURLODTNode('http://mbglrenderer/render')
         collection = GeometryCollection(LineString([[0, -85.06], [0, 85.06]]), srid=4326)
         self.assertEqual(0, node.get_zoom_bounds(1024, 1024, collection))
 
     def test_get_zoom_no_height(self):
-        node = MapImageLoaderNodeURL('http://mbglrenderer/render')
+        node = MapImageLoaderURLODTNode('http://mbglrenderer/render')
         collection = GeometryCollection(LineString([[-180, 0], [180, 0]]), srid=4326)
         self.assertEqual(0, node.get_zoom_bounds(1024, 1024, collection))
 
@@ -379,3 +426,44 @@ class StoredBase64FileTestCase(APITestCase):
         self.feature.refresh_from_db()
         data = stored_image_base64(self.feature.properties['logo'])
         self.assertTrue(data.startswith("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42mNk+M9QDwADhgGAWjR9"))
+
+
+@override_settings(MEDIA_ROOT=TemporaryDirectory().name)
+class PictogramURLForValueTestCase(TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        cls.picture_name = 'small_picture.png'
+        cls.crud_view = factories.CrudViewFactory()
+        cls.property = CrudViewProperty.objects.create(view=cls.crud_view, key="gender",
+                                                       json_schema={'type': "string",
+                                                                    "title": "Gender", })
+        CrudViewProperty.objects.create(view=cls.crud_view, key="other",
+                                        json_schema={'type': "string",
+                                                     "title": "Other", })
+        with open(os.path.join(PICTURES_PATH, cls.picture_name), 'rb') as picto:
+            PropertyEnum.objects.create(
+                value="M",
+                pictogram=ContentFile(picto.read(), name=cls.picture_name),
+                property=cls.property
+            )
+        sync_layer_schema(cls.crud_view)
+        cls.feature = Feature.objects.create(
+            layer=cls.crud_view.layer,
+            geom="POINT(0 0)",
+            properties={
+                "gender": "M"
+            }
+        )
+
+    def test_rendering_with_picto(self):
+        context = Context({'object': self.feature})
+        template_to_render = Template('{% load map_tags %}{{ object|get_pictogram_url_for_value:"gender" }}')
+        rendered_template = template_to_render.render(context)
+        self.assertTrue(rendered_template.startswith(settings.MEDIA_URL))
+        self.assertTrue(rendered_template.endswith(self.picture_name))
+
+    def test_rendering_without_picto(self):
+        context = Context({'object': self.feature})
+        template_to_render = Template('{% load map_tags %}{{ object|get_pictogram_url_for_value:"other" }}')
+        rendered_template = template_to_render.render(context)
+        self.assertEqual(rendered_template, "")
