@@ -1,6 +1,7 @@
 from tempfile import TemporaryDirectory
 
 from django.contrib.gis.geos import Point
+from django.core.files.uploadedfile import SimpleUploadedFile
 from django.test import TestCase
 from django.test.utils import override_settings
 from django.urls import reverse
@@ -8,9 +9,11 @@ from geostore import GeometryTypes
 from geostore.models import Feature, LayerExtraGeom, FeatureExtraGeom
 from rest_framework import status
 from rest_framework.test import APITestCase
-from terra_accounts.tests.factories import TerraUserFactory
+from terra_geocrud.properties.schema import sync_layer_schema
 
-from terra_geocrud.models import CrudViewProperty
+from terra_geocrud.tests.factories import AttachmentCategoryFactory, UserFactory
+
+from terra_geocrud.models import CrudViewProperty, PropertyEnum
 from . import factories
 from .settings import FEATURE_PROPERTIES, LAYER_SCHEMA
 from .. import models, settings as app_settings
@@ -164,6 +167,51 @@ class CrudViewSetTestCase(APITestCase):
         group_1.delete()
         group_2.delete()
 
+    def test_json_schema(self):
+        prop_country = CrudViewProperty.objects.create(view=self.view_1, key="country",
+                                                       json_schema={'type': "string", "title": "Country"})
+        prop_themes = CrudViewProperty.objects.create(view=self.view_1, key="themes",
+                                                      json_schema={'type': "array", "items": {"type": "string"}})
+        PropertyEnum.objects.create(value="France", property=prop_country)
+        PropertyEnum.objects.create(value="Spain", property=prop_country)
+        PropertyEnum.objects.create(value="Theme 1", property=prop_themes)
+        PropertyEnum.objects.create(value="Theme 2", property=prop_themes)
+        sync_ui_schema(self.view_1)
+        sync_layer_schema(self.view_1)
+        response = self.client.get(reverse('crudview-detail', args=(self.view_1.pk,)))
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        data = response.json()
+        self.assertDictEqual(
+            data['form_schema'],
+            {'properties': {'country': {'enum': ['Spain', 'France'],
+                                        'title': 'Country',
+                                        'type': 'string'},
+                            "themes": {"type": "array",
+                                       "items": {
+                                           "type": "string",
+                                           "enum": ["Theme 2", "Theme 1"]
+                                       }}},
+             'required': []})
+
+    def test_property_enum_are_casted(self):
+        prop_1 = CrudViewProperty.objects.create(view=self.view_1, key="age2",
+                                                 json_schema={'type': "integer", "title": "Age"})
+        prop_2 = CrudViewProperty.objects.create(view=self.view_1, key="height2",
+                                                 json_schema={'type': "number", "title": "Height"})
+        PropertyEnum.objects.create(value="1", property=prop_1)
+        PropertyEnum.objects.create(value="2", property=prop_1)
+        PropertyEnum.objects.create(value="1.1", property=prop_2)
+        PropertyEnum.objects.create(value="2", property=prop_2)
+        sync_ui_schema(self.view_1)
+        sync_layer_schema(self.view_1)
+        response = self.client.get(reverse('crudview-detail', args=(self.view_1.pk,)))
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        data = response.json()
+        self.assertListEqual(sorted(data['form_schema']['properties']['age2']['enum']),
+                             sorted([1, 2]))
+        self.assertListEqual(sorted(data['form_schema']['properties']['height2']['enum']),
+                             sorted([1.1, 2]))
+
 
 @override_settings(MEDIA_ROOT=TemporaryDirectory().name)
 class CrudSettingsViewTestCase(TestCase):
@@ -173,7 +221,7 @@ class CrudSettingsViewTestCase(TestCase):
         self.view_1 = factories.CrudViewFactory(name="View 1", order=0, group=self.group_1)
         self.view_2 = factories.CrudViewFactory(name="View 2", order=0, group=self.group_2)
         self.view_3 = factories.CrudViewFactory(name="View 3", order=1, group=self.group_2)
-        self.response = self.client.get(reverse('settings'))
+        self.response = self.client.get(reverse('crud-settings'))
 
     def test_endpoint_access(self):
         self.assertEqual(self.response.status_code, status.HTTP_200_OK)
@@ -201,22 +249,37 @@ class CrudRenderPointTemplateDetailViewTestCase(APITestCase):
             geom=Point(x=-0.246322800072846, y=44.5562461167907),
             properties=FEATURE_PROPERTIES,
         )
-        self.template = factories.TemplateDocxFactory.create(
-            name='Template',
+        self.template_odt = factories.TemplateDocxFactory.create(
+            name='Template ODT',
         )
-        self.crud_view.templates.add(self.template)
+        self.template_pdf = factories.TemplatePDFFactory.create(
+            name='Template PDF',
+        )
+        self.crud_view.templates.add(self.template_odt)
+        self.crud_view.templates.add(self.template_pdf)
 
-    def test_template_rendering(self):
+    def test_template_rendering_odt(self):
         response = self.client.get(
             reverse(
                 'feature-generate-template',
-                args=(self.feature.layer.pk, self.feature.identifier, self.template.pk)
+                args=(self.feature.layer.pk, self.feature.identifier, self.template_odt.pk)
             )
         )
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response._headers['content-type'][-1],
+        self.assertEqual(response['content-type'],
                          'application/vnd.openxmlformats-officedocument.wordprocessingml.document')
 
+    def test_template_rendering_pdf(self):
+        response = self.client.get(
+            reverse(
+                'feature-generate-template',
+                args=(self.feature.layer.pk, self.feature.identifier, self.template_pdf.pk)
+            )
+        )
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response['content-type'],
+                         'application/pdf')
+
 
 @override_settings(MEDIA_ROOT=TemporaryDirectory().name)
 class CrudFeatureViewsSetTestCase(APITestCase):
@@ -261,7 +324,7 @@ class CrudFeatureViewsSetTestCase(APITestCase):
         self.attachments = factories.FeatureAttachmentFactory.create_batch(10, feature=self.feature)
         self.template = factories.TemplateDocxFactory()
         self.crud_view.templates.add(self.template)
-        self.user = TerraUserFactory()
+        self.user = UserFactory()
         self.client.force_authenticate(self.user)
 
     def test_list_endpoint(self):
@@ -322,3 +385,23 @@ class CrudFeatureViewsSetTestCase(APITestCase):
         response = self.client.get(reverse('picture-list',
                                            args=(self.feature.identifier, )))
         self.assertEqual(response.status_code, status.HTTP_200_OK)
+
+
+@override_settings(MEDIA_ROOT=TemporaryDirectory().name)
+class FeatureAttachmentViewsetTesCase(APITestCase):
+    def setUp(self) -> None:
+        self.crud_view = factories.CrudViewFactory()
+        self.feature = Feature.objects.create(layer=self.crud_view.layer,
+                                              geom='POINT(0 0)')
+        self.category = AttachmentCategoryFactory()
+        self.user = UserFactory()
+        self.client.force_authenticate(self.user)
+
+    def test_feature_is_not_required_in_attachment_creation(self):
+        file = SimpleUploadedFile("file.jpg", b"file_content", content_type="image/jpg")
+        response = self.client.post(reverse('attachment-list',
+                                            args=(self.feature.identifier, )),
+                                    {"category": self.category.pk,
+                                     "legend": "file_test",
+                                     "file": file}, format='multipart')
+        self.assertEqual(response.status_code, 201, response.json())
